/*
 * Copyright 2007 Gabe Dijkstra and Robert Hensing
 *
 * This file is part of the Huygens Raytracer.
 * The Huygens Raytracer is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * The Huygens Raytracer is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * The GNU General Public License can be found in a file called LICENSE
 * in the directory where this file resides. It is also available at
 * http://www.gnu.org/licenses/
 *
 */

#ifndef Camera_h
#define Camera_h

#include "Math.h"

class Scene;
class Color;

class Camera {
protected:
  Scene *scene;
  Vector position;
  Matrix3 *direction_transform;
  int max_depth;
  float max_light;
  
public: 
  Camera() : scene(NULL), direction_transform(NULL), max_depth(4), max_light(INFINITY) {};
  Camera(Scene *, Vector position);
  virtual ~Camera();
  
  virtual Color raytracePixel(int x, int y) = 0;
  void setDirectionTransformation(Matrix3);
  Scene *getScene();
  void setPosition(Vector);
  void setMaxDepth(int);
  void setMaxLight(float);
  virtual void setImageDimensions(int, int) = 0;
};

class PlanarCamera : public Camera {
  float width, inv_width, half_width, half_height;
  float dov;
  int n;

public:
  PlanarCamera(Scene *, const char *);
  Color raytracePixel(int x, int y);
  void setImageDimensions(int, int);
};

class SphericCamera : public Camera {
  float x_start;
  float x_end;
  float x_step;
  
  float y_start;
  float y_end;
  float y_step;
  
  int n;
  
public:
  SphericCamera(Scene *, const char *);
  Color raytracePixel(int x, int y);
  void setImageDimensions(int, int);
};

class CylindricCamera : public Camera {
  float x_start;
  float x_end;
  float x_step;
  
  float y_start;
  float y_step;
  float cyl_offset;
  
  int n;
  
public:
  CylindricCamera(Scene *, const char *);
  Color raytracePixel(int x, int y);
  void setImageDimensions(int, int);
};

#endif
