/*
 * Copyright 2007 Gabe Dijkstra and Robert Hensing
 *
 * This file is part of the Huygens Raytracer.
 * The Huygens Raytracer is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * The Huygens Raytracer is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * The GNU General Public License can be found in a file called LICENSE
 * in the directory where this file resides. It is also available at
 * http://www.gnu.org/licenses/
 *
 */

#include "Color.h"

#include <stdio.h>
#include <math.h>

void
Color::clamp (void) {
#define cl(a) \
  if (a < 0.0f) \
    a = 0.0f; \
  if (a > 1.0f) \
    a = 1.0f
  cl(r);
  cl(g);
  cl(b);
#undef cl
}

void
Color::clamp (float c) {
#define cl(a) \
  if (a < 0.0f) \
    a = 0.0f; \
  if (a > c) \
    a = c;
  cl(r);
  cl(g);
  cl(b);
#undef cl
}

Color
Color::operator*(float a) const {
  Color ret;

  ret.r = r * a;
  ret.g = g * a;
  ret.b = b * a;

  return ret;
}

Color &
Color::operator^=(float a) {
  r = powf(r, a);
  g = powf(g, a);
  b = powf(b, a);
  
  return *this;
}

Color
Color::operator^(float a) const {
  Color ret(powf(r, a),
	    powf(g, a),
	    powf(b, a));
  return ret;
}

Color
Color::operator*(Color a) const {
  Color ret;

  ret.r = r * a.r;
  ret.g = g * a.g;
  ret.b = b * a.b;

  return ret;
}

Color
Color::operator+(Color c) const {
  Color n;

  n.r = c.r + this->r;
  n.g = c.g + this->g;
  n.b = c.b + this->b;

  return n;
}

Color&
Color::operator+=(Color c) {
  r += c.r;
  g += c.g;
  b += c.b;

  return *this;
}
