/*
 * Copyright 2007 Gabe Dijkstra and Robert Hensing
 *
 * This file is part of the Huygens Raytracer.
 * The Huygens Raytracer is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * The Huygens Raytracer is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * The GNU General Public License can be found in a file called LICENSE
 * in the directory where this file resides. It is also available at
 * http://www.gnu.org/licenses/
 *
 */

#ifndef Color_h
#define Color_h

class Color {
 public:
  float r,g,b;
  
  Color (void): r(0.0f), g(0.0f), b(0.0f) {}
  Color (float red, float green, float blue): r(red), g(green), b(blue) {}
  Color (float lum): r(lum), g(lum), b(lum) {}

  Color operator*(float) const;
  Color operator*(Color) const;
  Color operator+(Color) const;
  Color operator^(float) const;
  inline Color& operator*=(float f) { r *= f; g *= f; b *= f; return *this; }
  inline Color& operator/=(float f) { r /= f; g /= f; b /= f; return *this; }
  Color& operator+=(Color);
  Color& operator^=(float);

  void clamp(void);
  void clamp(float);
};

#endif
