/*
 * Copyright 2007 Gabe Dijkstra and Robert Hensing
 *
 * This file is part of the Huygens Raytracer.
 * The Huygens Raytracer is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * The Huygens Raytracer is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * The GNU General Public License can be found in a file called LICENSE
 * in the directory where this file resides. It is also available at
 * http://www.gnu.org/licenses/
 *
 */

#ifndef Scene_h
#define Scene_h

#include <vector>
#include "Color.h"
#include "Math.h"

class Object;

class Light {
public:
  Vector position; // direction in case of nonpositional light
  Color color;
  bool positional;
  
  Light(void);
  Light(Vector);
  Light(Vector, Color);
  Light(const char *desc);
};

class Scene {
  Color background;
  Color ambient;
  std::vector<Object *>objects;
  std::vector<Light *>lights;
  
public:
  Scene(void);
  ~Scene(void);
  
  void setAmbient(Color);
  void setBackground(Color);
  void add (Object *);
  void add (Light *);
  Color raytrace (Ray ray, int recursion_depth, Object *exclude);
  
  Color shade(Object *object, Color, Ray, Vector, Vector, int);
};

#endif
