/*
 * Copyright 2007 Gabe Dijkstra and Robert Hensing
 *
 * This file is part of the Huygens Raytracer.
 * The Huygens Raytracer is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * The Huygens Raytracer is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * The GNU General Public License can be found in a file called LICENSE
 * in the directory where this file resides. It is also available at
 * http://www.gnu.org/licenses/
 *
 */

#include "Triangle.h"
#include "Scene.h"
#include <stdio.h>

Triangle::Triangle(void) {
  
}

Triangle::Triangle(Vector v0, Vector v1, Vector v2) {

}

Triangle::Triangle(Vector v0, Vector v1, Vector v2, Color c) {

}

Triangle::Triangle(const char *desc) {
  float x0,y0,z0, x1, y1, z1, x2, y2, z2,
    R = 1.0f, G = 1.0f, B = 1.0f;

  diffuse_coeff = 1.0,
  specular_coeff = 0.25,
  shininess = 5.0;
  reflectivity = 0.0;
  
  specular_coeff = 0.0f;

  int r = sscanf(desc, "\
%*s (%f, %f, %f) (%f, %f, %f) (%f, %f, %f) {%f, %f, %f} %f %f %f %f",
		 &x0, &y0, &z0, &x1, &y1, &z1, &x2, &y2, &z2,
		 &R, &G, &B,
		 &diffuse_coeff, &specular_coeff, &shininess, &reflectivity);

  if (r == 9 || (r >= 12 && r <= 16)) {
    color = Color(R, G, B);
    vertices[0] = Vector(x0, y0, z0);
    vertices[1] = Vector(x1, y1, z1);
    vertices[2] = Vector(x2, y2, z2);

    normal = crossproduct(vertices[1] - vertices[0],
			  vertices[2] - vertices[0]);
    normal.normalize();
  }
  else {
    throw (void*)NULL;
  }
}

float Triangle::intersect(Ray ray) {
  float a = vertices[0].x - vertices[1].x;
  float b = vertices[0].y - vertices[1].y;
  float c = vertices[0].z - vertices[1].z;

  float d = vertices[0].x - vertices[2].x;
  float e = vertices[0].y - vertices[2].y;
  float f = vertices[0].z - vertices[2].z;

  float g = ray.direction.x;
  float h = ray.direction.y;
  float i = ray.direction.z;

  float j = vertices[0].x - ray.origin.x;
  float k = vertices[0].y - ray.origin.y;
  float l = vertices[0].z - ray.origin.z;

  float invM = 1.0 / (a*(e*i - h*f) + b*(g*f - d*i) + c*(d*h - e*g));
  
  float beta = (j*(e*i - h*f) + k*(g*f - d*i) + l*(d*h - e*g)) * invM;
  
  if (beta < 0.0 || beta > 1)
    return INFINITY;

  float gamma = (i*(a*k - j*b) + h*(j*c - a*l) + g*(b*l - k*c)) * invM;

  if (gamma < 0.0 || gamma > (1.0 - beta))
    return INFINITY;

  float t = - (f*(a*k - j*b) + e*(j*c - a*l) + d*(b*l - k*c)) * invM;

  if (t < 0)
    return INFINITY;
  else
    return t;
}

Color Triangle::reflect(Ray ray, float t, int recur_depth) {
  Vector point = ray.origin + ray.direction * t;
  Ray refl_ray;
  
  refl_ray.origin = point;
  refl_ray.direction = normal * (-2 * dotproduct(ray.direction, normal)) + ray.direction;
  
  if (reflectivity == 0.0f)
    return scene->shade(this, color, ray, point, normal, recur_depth);
  else 
    return scene->shade(this, color, ray, point, normal, recur_depth) * (1.0 - reflectivity)
      + color * scene->raytrace(refl_ray, recur_depth, this) * reflectivity;
}
